@@ -28,10 +28,14 @@
 #include <giomm.h>
 
 #include "application.h"
+
+#ifdef HAVE_BRASERO
+#include "brasero-destination.h"
+#endif // HAVE_BRASERO
+
 #include "browser-model-column-record.h"
 #include "browser-renderer.h"
 //#include "camera-source.h"
-//#include "console-renderer.h"
 #include "content-type-repo.h"
 //#include "date-manager.h"
 #include "directory-destination.h"
@@ -44,9 +48,8 @@
 #include "i-plugin.h"
 //#include "importer.h"
 #include "photo.h"
-#include "progress-observer.h"
 #include "property-manager.h"
-#include "search-basket.h"
+#include "search-manager.h"
 #include "slideshow-renderer.h"
 #include "tag-manager.h"
 #include "thumbnailer.h"
@@ -56,7 +59,7 @@ namespace Solang
 
 template<typename T>
 class Initializer :
-    public std::unary_function<std::pair<const std::string, T> &, void>
+    public std::unary_function<T &, void>
 {
     public:
         Initializer(Application * const & application) throw();
@@ -69,7 +72,7 @@ class Initializer :
         operator=(const Initializer<T> & source) throw();
 
         void
-        operator()(std::pair<const std::string, T> & x) throw();
+        operator()(T & x) throw();
 
     protected:
         Application * application_;
@@ -77,14 +80,14 @@ class Initializer :
 
 template<typename T>
 Initializer<T>::Initializer(Application * const & application) throw() :
-    std::unary_function<std::pair<const std::string, T> &, void>(),
+    std::unary_function<T &, void>(),
     application_(application)
 {
 }
 
 template<typename T>
 Initializer<T>::Initializer(const Initializer<T> & source) throw() :
-    std::unary_function<std::pair<const std::string, T> &, void>(source),
+    std::unary_function<T &, void>(source),
     application_(source.application_)
 {
 }
@@ -100,8 +103,7 @@ Initializer<T>::operator=(const Initializer<T> & source) throw()
 {
     if (this != &source)
     {
-        std::unary_function<std::pair<const std::string, T> &, void>
-            ::operator=(source);
+        std::unary_function<T &, void>::operator=(source);
         application_ = source.application_;
     }
 
@@ -110,14 +112,14 @@ Initializer<T>::operator=(const Initializer<T> & source) throw()
 
 template<typename T>
 void
-Initializer<T>::operator()(std::pair<const std::string, T> & x) throw()
+Initializer<T>::operator()(T & x) throw()
 {
-    x.second->init(*application_);
+    x->init(*application_);
 }
 
 template<typename T>
 class Finalizer :
-    public std::unary_function<std::pair<const std::string, T> &, void>
+    public std::unary_function<T &, void>
 {
     public:
         Finalizer(Application * const & application) throw();
@@ -130,7 +132,7 @@ class Finalizer :
         operator=(const Finalizer<T> & source) throw();
 
         void
-        operator()(std::pair<const std::string, T> & x) throw();
+        operator()(T & x) throw();
 
     protected:
         Application * application_;
@@ -138,14 +140,14 @@ class Finalizer :
 
 template<typename T>
 Finalizer<T>::Finalizer(Application * const & application) throw() :
-    std::unary_function<std::pair<const std::string, T> &, void>(),
+    std::unary_function<T &, void>(),
     application_(application)
 {
 }
 
 template<typename T>
 Finalizer<T>::Finalizer(const Finalizer<T> & source) throw() :
-    std::unary_function<std::pair<const std::string, T> &, void>(source),
+    std::unary_function<T &, void>(source),
     application_(source.application_)
 {
 }
@@ -161,8 +163,7 @@ Finalizer<T>::operator=(const Finalizer<T> & source) throw()
 {
     if (this != &source)
     {
-        std::unary_function<std::pair<const std::string, T> &, void>
-            ::operator=(source);
+        std::unary_function<T &, void>::operator=(source);
         application_ = source.application_;
     }
 
@@ -171,19 +172,18 @@ Finalizer<T>::operator=(const Finalizer<T> & source) throw()
 
 template<typename T>
 void
-Finalizer<T>::operator()(std::pair<const std::string, T> & x) throw()
+Finalizer<T>::operator()(T & x) throw()
 {
-    x.second->final(*application_);
+    x->final(*application_);
 }
 
 Application::Application(int & argc, char ** & argv) throw() :
     sigc::trackable(),
     threadPool_(4, false),
     iconFactory_(Gtk::IconFactory::create()),
-    observer_( new ProgressObserver() ),
-    engine_(argc, argv, observer_),
+    engine_(argc, argv),
     mainWindow_(),
-    progressDialog_(engine_.get_default_observer()),
+    progressDialog_(mainWindow_),
     listStore_(Gtk::ListStore::create(BrowserModelColumnRecord())),
     listStoreIter_(),
     plugins_(),
@@ -243,19 +243,11 @@ Application::Application(int & argc, char ** & argv) throw() :
         }
     }
 
-    add_icons();
+    Gtk::Window::set_default_icon_name(PACKAGE_TARNAME);
 
     engine_.signal_criteria_changed().connect(
         sigc::mem_fun(*this,
                       &Application::on_criteria_changed));
-    engine_.photo_export_begin().connect(sigc::mem_fun(*this,
-        &Application::show_progress_dialog));
-    engine_.photo_export_end().connect(sigc::mem_fun(*this,
-        &Application::hide_progress_dialog));
-    engine_.photo_import_begin().connect(sigc::mem_fun(*this,
-        &Application::show_progress_dialog));
-    engine_.photo_import_end().connect(sigc::mem_fun(*this,
-        &Application::hide_progress_dialog));
 
     Thumbnailer & thumbnailer = Thumbnailer::instance();
     thumbnailer.signal_ready().connect(
@@ -266,6 +258,9 @@ Application::Application(int & argc, char ** & argv) throw() :
 Application::~Application() throw()
 {
     threadPool_.shutdown(false);
+
+    Thumbnailer & thumbnailer = Thumbnailer::instance();
+    thumbnailer.shutdown();
 }
 
 void
@@ -273,7 +268,7 @@ Application::init() throw()
 {
     engine_.init("");
     engine_.search_async(
-        PhotoSearchCriteriaList(),
+        IPhotoSearchCriteriaList(),
         sigc::mem_fun(*this,
                       &Application::on_async_search));
 
@@ -284,56 +279,56 @@ Application::init() throw()
 //                                   date_manager));
 
     IPluginPtr editor(new Editor());
-    plugins_.insert(std::make_pair("editor", editor));
+    plugins_.push_back(editor);
 
     IPluginPtr property_manager(new PropertyManager());
-    plugins_.insert(std::make_pair("property-manager",
-                                   property_manager));
+    plugins_.push_back(property_manager);
 
-    IPluginPtr search_basket(new SearchBasket());
-    plugins_.insert(std::make_pair("search-basket", search_basket));
+    IPluginPtr search_manager(new SearchManager());
+    plugins_.push_back(search_manager);
 
     IPluginPtr tag_manager(new TagManager( ));
-    plugins_.insert(std::make_pair("tag-manager", tag_manager));
+    plugins_.push_back(tag_manager);
 
     IPhotoDestinationPtr directory_destination(
                              new DirectoryDestination());
     IPluginPtr directory_exporter(new Exporter(directory_destination,
                                                true));
-    plugins_.insert(std::make_pair("directory-exporter",
-                                   directory_exporter));
+    plugins_.push_back(directory_exporter);
+
+#ifdef HAVE_BRASERO
+    IPhotoDestinationPtr brasero_destination(
+                             new BraseroDestination());
+    IPluginPtr brasero_exporter(new Exporter(brasero_destination,
+                                             false));
+    plugins_.push_back(brasero_exporter);
+#endif // HAVE_BRASERO
 
 //    IPhotoSourcePtr directory_source(new DirectorySource());
 //    IPluginPtr directory_importer(new Importer(directory_source, true));
-//    plugins_.insert(std::make_pair("directory-importer",
-//                                   directory_importer));
+//    plugins_.push_back(directory_importer);
 
 //    IPhotoSourcePtr camera_source(new CameraSource());
 //    IPluginPtr camera_importer(new Importer(camera_source, false));
-//    plugins_.insert(std::make_pair("camera-importer", camera_importer));
+//    plugins_.push_back(camera_importer);
 
 //    IPhotoSourcePtr flickr_source(new FlickrSource());
 //    IPluginPtr flickr_importer(new Importer(flickr_source, false));
-//    plugins_.insert(std::make_pair("flickr-importer", flickr_importer));
-
-    std::for_each(plugins_.begin(), plugins_.end(),
-                  Initializer<IPluginPtr>(this));
+//    plugins_.push_back(flickr_importer);
 
     // Renderers.
 
-    IRendererPtr browser_renderer(new BrowserRenderer());
-    rendererRegistry_.add(browser_renderer);
+    IPluginPtr browser_renderer(new BrowserRenderer());
+    plugins_.push_back(browser_renderer);
 
-//    IRendererPtr console_renderer(new ConsoleRenderer());
-//    rendererRegistry_.add(console_renderer);
+    IPluginPtr enlarged_renderer(new EnlargedRenderer());
+    plugins_.push_back(enlarged_renderer);
 
-    IRendererPtr enlarged_renderer(new EnlargedRenderer());
-    rendererRegistry_.add(enlarged_renderer);
+    IPluginPtr slideshow_renderer(new SlideshowRenderer());
+    plugins_.push_back(slideshow_renderer);
 
-    IRendererPtr slideshow_renderer(new SlideshowRenderer());
-    rendererRegistry_.add(slideshow_renderer);
-
-    rendererRegistry_.init(*this);
+    std::for_each(plugins_.begin(), plugins_.end(),
+                  Initializer<IPluginPtr>(this));
 
     const IRendererPtr renderer
         = rendererRegistry_.select<BrowserRenderer>();
@@ -347,8 +342,6 @@ Application::init() throw()
 
     mainWindow_.init(*this);
 
-    mainWindow_.connect_progress( engine_.get_default_observer() );
-
     initEnd_.emit(*this);
 }
 
@@ -365,54 +358,11 @@ Application::final() throw()
     engine_.final();
     mainWindow_.final(*this);
 
-    rendererRegistry_.final(*this);
-
-    std::for_each(plugins_.begin(), plugins_.end(),
+    std::for_each(plugins_.rbegin(), plugins_.rend(),
                   Finalizer<IPluginPtr>(this));
     plugins_.clear();
 }
 
-void
-Application::add_icons() throw()
-{
-    Gtk::IconSource icon_source;
-    Gtk::IconSet icon_set_slideshow_play;
-
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME
-            "/pixmaps/slideshow-play-16.png");
-    icon_source.set_size(Gtk::IconSize(16));
-    icon_set_slideshow_play.add_source(icon_source);
-
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME
-            "/pixmaps/slideshow-play-22.png");
-    icon_source.set_size(Gtk::IconSize(22));
-    icon_set_slideshow_play.add_source(icon_source);
-
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME
-            "/pixmaps/slideshow-play-24.png");
-    icon_source.set_size(Gtk::IconSize(24));
-    icon_set_slideshow_play.add_source(icon_source);
-
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME
-            "/pixmaps/slideshow-play-32.png");
-    icon_source.set_size(Gtk::IconSize(32));
-    icon_set_slideshow_play.add_source(icon_source);
-
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME
-            "/pixmaps/slideshow-play-48.png");
-    icon_source.set_size(Gtk::IconSize(48));
-    icon_set_slideshow_play.add_source(icon_source);
-
-    iconFactory_->add(Gtk::StockID(PACKAGE_TARNAME"-slideshow-play"),
-                      icon_set_slideshow_play);
-    iconFactory_->add_default();
-}
-
 void
 Application::add_photo_to_model(const PhotoPtr & photo) throw()
 {
@@ -466,13 +416,6 @@ Application::init_end() throw()
     return initEnd_;
 }
 
-void
-Application::hide_progress_dialog() throw()
-{
-    progressDialog_.hide();
-    engine_.get_default_observer()->reset();
-}
-
 sigc::signal<void, Application &> &
 Application::list_store_change_begin() throw()
 {
@@ -494,7 +437,7 @@ Application::on_async_search(PhotoList & photos) throw()
 }
 
 void
-Application::on_criteria_changed(PhotoSearchCriteriaList & criteria)
+Application::on_criteria_changed(IPhotoSearchCriteriaList & criteria)
                                  throw()
 {
     engine_.search_async(
@@ -543,14 +486,6 @@ Application::on_thumbnailer_ready(PhotoList & photos) const throw()
     }
 }
 
-void
-Application::show_progress_dialog() throw()
-{
-    progressDialog_.set_transient_for(mainWindow_);
-    progressDialog_.show_all();
-}
-
-
 Glib::ThreadPool &
 Application::get_thread_pool() throw()
 {
@@ -569,6 +504,12 @@ Application::get_main_window() throw()
     return mainWindow_;
 }
 
+ProgressDialog &
+Application::get_progress_dialog() throw()
+{
+    return progressDialog_;
+}
+
 const ListStorePtr &
 Application::get_list_store() throw()
 {
@@ -609,7 +550,7 @@ Application::get_drag_drop_map() throw()
 
 void
 Application::set_drag_item(const Glib::ustring & key,
-                const PhotoSearchCriteriaPtr &criteria) throw()
+                const IPhotoSearchCriteriaPtr &criteria) throw()
 {
     //Existing item will be replaced
     //This handles double dragging of same criteria
