@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
- * Copyright (C) 2009 Debarshi Ray <rishi@gnu.org>
+ * Copyright (C) 2009, 2010 Debarshi Ray <rishi@gnu.org>
  *
  * Solang is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,6 +19,7 @@
 #ifndef SOLANG_MAIN_WINDOW_H
 #define SOLANG_MAIN_WINDOW_H
 
+#include <string>
 #include <vector>
 
 #include <gdl/gdl.h>
@@ -36,7 +37,7 @@ class Application;
 
 class MainWindow :
     public Gtk::Window,
-    public NonCopyable
+    private NonCopyable
 {
     public:
         MainWindow() throw();
@@ -80,15 +81,13 @@ class MainWindow :
         void
         set_busy(bool busy) throw();
 
-        void
-        connect_progress(
-                const ProgressObserverPtr &observer ) throw();
-
-
     protected:
         std::string
         get_user_layout_file() throw();
 
+        void
+        load_layout() throw();
+
         void
         on_about_activate_link_url(Gtk::AboutDialog & about_dialog,
                                    const Glib::ustring & link)
@@ -103,6 +102,9 @@ class MainWindow :
         void
         on_action_help_about() throw();
 
+        void
+        on_action_help_contents() throw();
+
         void
         on_action_photo_quit() throw();
 
@@ -120,10 +122,7 @@ class MainWindow :
         on_delete_event(GdkEventAny * event);
 
         void
-        on_progress() throw();
-
-        void
-        on_reset() throw();
+        save_layout() throw();
 
         ApplicationPtr application_;
 
@@ -141,10 +140,6 @@ class MainWindow :
 
         Gtk::Statusbar statusBar_;
 
-        Gtk::ProgressBar progress_;
-
-        ProgressObserverPtr observer_;
-
         GtkWidget * const dock_;
 
         GtkWidget * const dockBar_;
@@ -162,6 +157,17 @@ class MainWindow :
         std::vector<DockObjectPtr> dockObjectsCenter_;
 
     private:
+        static const std::string artistsFile_;
+
+        static const std::string authorsFile_;
+
+        static const std::string copyingFile_;
+
+        static const std::string layoutFile_;
+
+        static const std::string uiFile_;
+
+        static const Glib::ustring websiteURL_;
 };
 
 } // namespace Solang
