@@ -1,5 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
+ * Copyright (C) 2010 Debarshi Ray <rishi@gnu.org>
  * Copyright (C) 2009 Santanu Sinha <santanu.sinha@gmail.com>
  *
  * Solang is free software: you can redistribute it and/or modify it
@@ -24,7 +25,6 @@
 
 #include "application.h"
 #include "browser-renderer.h"
-#include "console-renderer.h"
 #include "date-manager.h"
 #include "engine.h"
 #include "enlarged-renderer.h"
@@ -35,7 +35,7 @@ namespace Solang
 {
 
 static const std::string uiFile
-    = PACKAGE_DATA_DIR"/"PACKAGE_TARNAME"/ui/"
+    = PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "ui" G_DIR_SEPARATOR_S
           PACKAGE_TARNAME"-property-manager.ui";
 
 DateManager::DateManager() throw() :
@@ -64,6 +64,8 @@ DateManager::DateManager() throw() :
 
     scrolledWindow_.set_policy(Gtk::POLICY_AUTOMATIC,
                                Gtk::POLICY_AUTOMATIC);
+    scrolledWindow_.set_shadow_type(Gtk::SHADOW_IN);
+
     scrolledWindow_.add(dateView_);
 
     vBox_.pack_start( scrolledWindow_, Gtk::PACK_EXPAND_WIDGET,0 );
@@ -116,13 +118,6 @@ DateManager::visit_renderer(BrowserRenderer & browser_renderer)
     ui_show();
 }
 
-void
-DateManager::visit_renderer(ConsoleRenderer & console_renderer)
-                            throw()
-{
-    ui_hide();
-}
-
 void
 DateManager::visit_renderer(EnlargedRenderer & enlarged_renderer)
                             throw()
