@@ -22,7 +22,6 @@
 #include <functional>
 
 #include <gdkmm.h>
-#include <glibmm.h>
 
 #include "types.h"
 
@@ -33,7 +32,7 @@ class ThumbbufMaker :
     public std::unary_function<const PhotoPtr &, PixbufPtr>
 {
     public:
-        ThumbbufMaker(guint width, guint height) throw();
+        ThumbbufMaker(guint width, guint height, bool rotate) throw();
 
         ThumbbufMaker(const ThumbbufMaker & source) throw();
 
@@ -43,11 +42,12 @@ class ThumbbufMaker :
         operator=(const ThumbbufMaker & source) throw();
 
         PixbufPtr
-        operator()(const PhotoPtr & photo) throw();
-
-    protected:
+        operator()(const PhotoPtr & photo) throw(Gdk::PixbufError,
+                                                 Glib::FileError);
 
     private:
+        bool rotate_;
+
         guint width_;
 
         guint height_;
