@@ -31,6 +31,7 @@
 #include "flip-vert-operation.h"
 #include "i-renderer.h"
 #include "non-copyable.h"
+#include "progress-observer.h"
 #include "rotate-clock-operation.h"
 #include "rotate-counter-operation.h"
 
@@ -38,7 +39,7 @@ namespace Solang
 {
 
 class IOperationFactory :
-    public NonCopyable
+    private NonCopyable
 {
     public:
         virtual
@@ -93,7 +94,7 @@ OperationFactory<T>::create() throw()
 }
 
 const std::string Editor::uiFile_
-    = PACKAGE_DATA_DIR"/"PACKAGE_TARNAME"/ui/"
+    = PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "ui" G_DIR_SEPARATOR_S
           PACKAGE_TARNAME"-editor.ui";
 
 Editor::Editor() throw() :
@@ -225,12 +226,6 @@ Editor::visit_renderer(BrowserRenderer & browser_renderer) throw()
     ui_hide();
 }
 
-void
-Editor::visit_renderer(ConsoleRenderer & browser_renderer) throw()
-{
-    ui_hide();
-}
-
 void
 Editor::visit_renderer(EnlargedRenderer & editor_renderer) throw()
 {
@@ -254,12 +249,8 @@ Editor::apply_async(const IOperationPtr & operation) throw()
 
     if (editablePhotos_.end() == iter)
     {
-        Engine & engine = application_->get_engine();
-        const ProgressObserverPtr & observer = engine.get_default_observer();
-
         editable_photo = EditablePhotoPtr(new EditablePhoto(
-                                              currentPhoto_,
-                                              observer));
+                                              currentPhoto_));
         editablePhotos_.insert(std::make_pair(current_uri,
                                               editable_photo));
     }
@@ -268,11 +259,16 @@ Editor::apply_async(const IOperationPtr & operation) throw()
         editable_photo = iter->second;
     }
 
+    const ProgressObserverPtr observer
+        = ProgressObserver<Glib::Dispatcher>::create(
+              application_->get_progress_dialog());
+
     editable_photo->apply_async(
         operation,
         sigc::bind(sigc::mem_fun(*this,
                                  &Editor::on_action_edit_photo_end),
-                   editable_photo));
+                   editable_photo),
+        observer);
 }
 
 void
