@@ -21,6 +21,8 @@
 #include "config.h"
 #endif // HAVE_CONFIG_H
 
+#include <algorithm>
+#include <functional>
 #include <sstream>
 #include <vector>
 
@@ -35,12 +37,72 @@
 #include "i-renderer-selector.h"
 #include "main-window.h"
 #include "photo.h"
-#include "photo-search-criteria.h"
 #include "scale-action.h"
 
 namespace Solang
 {
 
+class ChildPathConverter :
+    public std::unary_function<const Gtk::TreePath &, Gtk::TreePath>
+{
+    public:
+        ChildPathConverter(
+            const TreeModelFilterPtr & tree_model_filter) throw();
+
+        ChildPathConverter(const ChildPathConverter & source) throw();
+
+        ~ChildPathConverter() throw();
+
+        ChildPathConverter &
+        operator=(const ChildPathConverter & source) throw();
+
+        Gtk::TreeModel::Path
+        operator()(const Gtk::TreeModel::Path & path) throw();
+
+    protected:
+        TreeModelFilterPtr treeModelFilter_;
+};
+
+ChildPathConverter::ChildPathConverter(
+                        const TreeModelFilterPtr & tree_model_filter)
+                        throw() :
+    std::unary_function<const Gtk::TreePath &, Gtk::TreePath>(),
+    treeModelFilter_(tree_model_filter)
+{
+}
+
+ChildPathConverter::ChildPathConverter(
+    const ChildPathConverter & source) throw() :
+    std::unary_function<const Gtk::TreePath &, Gtk::TreePath>(source),
+    treeModelFilter_(source.treeModelFilter_)
+{
+}
+
+ChildPathConverter::~ChildPathConverter() throw()
+{
+}
+
+ChildPathConverter &
+ChildPathConverter::operator=(const ChildPathConverter & source)
+                              throw()
+{
+    if (this != &source)
+    {
+        std::unary_function<const Gtk::TreePath &,
+                            Gtk::TreePath>::operator=(source);
+        treeModelFilter_ = source.treeModelFilter_;
+    }
+
+    return *this;
+}
+
+Gtk::TreeModel::Path
+ChildPathConverter::operator()(const Gtk::TreeModel::Path & path)
+                               throw()
+{
+    return treeModelFilter_->convert_path_to_child_path(path);
+}
+
 static const guint lowerZoomValue = 20;
 static const guint higherZoomValue = 100;
 static const guint initialZoomValue
@@ -48,26 +110,26 @@ static const guint initialZoomValue
                          + (higherZoomValue - lowerZoomValue) / 2;
 static const guint stepZoomValue = 5;
 
-static const gint thumbnailRendererWidth = 168;
-static const gint thumbnailRendererHeight = 130;
+static const gint thumbnailRendererWidth = 156;
+static const gint thumbnailRendererHeight = 118;
 
 static const double ratioWidth
-    = static_cast<double>(thumbnailRendererWidth - 12)
-      / static_cast<double>(initialZoomValue);
+                        = static_cast<double>(thumbnailRendererWidth)
+                          / static_cast<double>(initialZoomValue);
 static const double ratioHeight
-    = static_cast<double>(thumbnailRendererHeight - 12)
-      / static_cast<double>(initialZoomValue);
+                        = static_cast<double>(thumbnailRendererHeight)
+                          / static_cast<double>(initialZoomValue);
 
 static const std::string uiFile
-    = PACKAGE_DATA_DIR"/"PACKAGE_TARNAME"/ui/"
+    = PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "ui" G_DIR_SEPARATOR_S
           PACKAGE_TARNAME"-browser-renderer.ui";
 static const std::string uiFileThumbnail
-    = PACKAGE_DATA_DIR"/"PACKAGE_TARNAME"/ui/"
+    = PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "ui" G_DIR_SEPARATOR_S
           PACKAGE_TARNAME"-browser-renderer-thumbnail.ui";
 
 BrowserRenderer::BrowserRenderer() throw() :
     IRenderer(),
-    sigc::trackable(),
+    Plugin(),
     application_(NULL),
     iconFactory_(Gtk::IconFactory::create()),
     actionGroup_(Gtk::ActionGroup::create(
@@ -99,26 +161,27 @@ BrowserRenderer::BrowserRenderer() throw() :
     Gtk::IconSource icon_source;
     Gtk::IconSet icon_set_mode_browse;
 
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME
-            "/pixmaps/mode-browse-16.png");
+    icon_source.set_filename(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S
+                             "pixmaps" G_DIR_SEPARATOR_S
+                             "mode-browse-16.png");
     icon_source.set_size(Gtk::IconSize(16));
     icon_set_mode_browse.add_source(icon_source);
 
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME
-            "/pixmaps/mode-browse-22.png");
+    icon_source.set_filename(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S
+                             "pixmaps" G_DIR_SEPARATOR_S
+                             "mode-browse-22.png");
     icon_source.set_size(Gtk::IconSize(22));
     icon_set_mode_browse.add_source(icon_source);
 
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME"/pixmaps/tag-new-24.png");
+    icon_source.set_filename(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S
+                             "pixmaps" G_DIR_SEPARATOR_S
+                             "tag-new-24.png");
     icon_source.set_size(Gtk::IconSize(24));
     icon_set_mode_browse.add_source(icon_source);
 
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME
-            "/pixmaps/mode-browse-32.png");
+    icon_source.set_filename(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S
+                             "pixmaps" G_DIR_SEPARATOR_S
+                             "mode-browse-32.png");
     icon_source.set_size(Gtk::IconSize(32));
     icon_set_mode_browse.add_source(icon_source);
 
@@ -138,9 +201,9 @@ BrowserRenderer::BrowserRenderer() throw() :
                       &BrowserRenderer::on_action_add_to_export_queue));
 
     actionGroup_->add(
-        Gtk::Action::create(
+        Gtk::Action::create_with_icon_name(
             "ActionViewBrowserSlideshow",
-            Gtk::StockID(PACKAGE_TARNAME"-slideshow-play"),
+            "slideshow-play",
             _("_Slideshow"),
             _("Start a slideshow view of the photos")),
         Gtk::AccelKey("F5"),
@@ -205,7 +268,8 @@ BrowserRenderer::BrowserRenderer() throw() :
             _("_Previous Page"),
             _("Go to the previous page in the collection"));
 
-        action->property_short_label().set_value(_("Previous"));
+        action->property_short_label().set_value(
+            C_("Navigation", "Previous"));
         action->property_is_important().set_value(true);
         action->set_sensitive(false);
         actionGroup_->add(
@@ -220,7 +284,8 @@ BrowserRenderer::BrowserRenderer() throw() :
             _("_Next Page"),
             _("Go to the next page in the collection"));
 
-        action->property_short_label().set_value(_("Next"));
+        action->property_short_label().set_value(
+            C_("Navigation", "Next"));
         action->property_is_important().set_value(true);
         actionGroup_->add(
             action, Gtk::AccelKey("<alt>Page_Down"),
@@ -253,6 +318,7 @@ BrowserRenderer::BrowserRenderer() throw() :
 
     scrolledWindow_.set_policy(Gtk::POLICY_AUTOMATIC,
                                Gtk::POLICY_AUTOMATIC);
+    scrolledWindow_.set_shadow_type(Gtk::SHADOW_IN);
 
     scrolledWindow_.add(thumbnailView_);
     vBox_.pack_start(scrolledWindow_, Gtk::PACK_EXPAND_WIDGET, 0);
@@ -303,6 +369,10 @@ BrowserRenderer::init(Application & application) throw()
 {
     application_ = &application;
 
+    RendererRegistry & renderer_registry
+        = application.get_renderer_registry();
+    renderer_registry.add(this);
+
     const ListStorePtr & list_store = application.get_list_store();
 
     treeModelFilter_ = Gtk::TreeModelFilter::create(list_store);
@@ -327,11 +397,6 @@ BrowserRenderer::init(Application & application) throw()
 
     ui_manager->insert_action_group(actionGroup_);
 
-    signalInitEnd_
-        = application.init_end().connect(
-              sigc::mem_fun(*this,
-                            &BrowserRenderer::on_init_end));
-
     // NB: This should not be done in the constructor because if
     //     'false == application_' then the handler will crash.
     //     Better safe than sorry.
@@ -358,16 +423,6 @@ BrowserRenderer::init(Application & application) throw()
     // initialized_.emit(*this);
 }
 
-void
-BrowserRenderer::render(const PhotoPtr & photo) throw()
-{
-}
-
-void
-BrowserRenderer::render(const PhotoList & photos) throw()
-{
-}
-
 void
 BrowserRenderer::final(Application & application) throw()
 {
@@ -389,9 +444,43 @@ BrowserRenderer::final(Application & application) throw()
 
     treeModelFilter_.reset();
 
+    main_window.undock_object_center(GDL_DOCK_OBJECT(dockItem_));
+
+    RendererRegistry & renderer_registry
+        = application.get_renderer_registry();
+    renderer_registry.remove(this);
+
     // finalized_.emit(*this);
 }
 
+void
+BrowserRenderer::visit_renderer(BrowserRenderer & browser_renderer)
+                                throw()
+{
+}
+
+void
+BrowserRenderer::visit_renderer(EnlargedRenderer & enlarged_renderer)
+                                throw()
+{
+}
+
+void
+BrowserRenderer::visit_renderer(SlideshowRenderer & slideshow_renderer)
+                                throw()
+{
+}
+
+void
+BrowserRenderer::render(const PhotoPtr & photo) throw()
+{
+}
+
+void
+BrowserRenderer::render(const PhotoList & photos) throw()
+{
+}
+
 void
 BrowserRenderer::clear_thumbnails() throw()
 {
@@ -423,6 +512,16 @@ BrowserRenderer::get_current_selection() throw()
     return thumbnailView_.get_selected_photos();
 }
 
+TreePathList
+BrowserRenderer::get_selected_paths() const throw()
+{
+    TreePathList paths = thumbnailView_.get_selected_items();
+    std::transform(paths.begin(), paths.end(),
+                   paths.begin(),
+                   ChildPathConverter(treeModelFilter_));
+    return paths;
+}
+
 void
 BrowserRenderer::present() throw()
 {
@@ -514,12 +613,6 @@ BrowserRenderer::on_action_view_slideshow() throw()
     slideshow_renderer->render(photo);
 }
 
-void
-BrowserRenderer::on_init_end(Application & application) throw()
-{
-    signalInitEnd_.disconnect();
-}
-
 void
 BrowserRenderer::on_selection_changed() throw()
 {
@@ -616,6 +709,16 @@ BrowserRenderer::on_limits_changed() throw()
 
     status_pop();
     status_push();
+
+    const Gtk::TreeModel::Children children
+                                       = treeModelFilter_->children();
+    if (true == children.empty())
+    {
+        return;
+    }
+
+    thumbnailView_.select_path(treeModelFilter_->get_path(
+                                   children.begin()));
 }
 
 void
@@ -775,17 +878,17 @@ BrowserRenderer::reload() throw()
 void
 BrowserRenderer::set_thumbnail_size() throw()
 {
-    const guint thumbnail_width
+    const guint thumbnail_renderer_width
                     = static_cast<guint>(
                           ratioWidth
                           * static_cast<double>(zoomValue_));
-    const guint thumbnail_height
+    const guint thumbnail_renderer_height
                     = static_cast<guint>(
                           ratioHeight
                           * static_cast<double>(zoomValue_));
 
-    thumbnailView_.set_thumbnail_width(thumbnail_width + 6);
-    thumbnailView_.set_thumbnail_height(thumbnail_height + 6);
+    thumbnailView_.set_thumbnail_width(thumbnail_renderer_width);
+    thumbnailView_.set_thumbnail_height(thumbnail_renderer_height);
 }
 
 void
