@@ -27,6 +27,7 @@
 #include <sigc++/sigc++.h>
 
 #include "i-renderer.h"
+#include "plugin.h"
 #include "thumbnail-view.h"
 #include "types.h"
 
@@ -35,7 +36,7 @@ namespace Solang
 
 class EnlargedRenderer :
     public IRenderer,
-    public sigc::trackable
+    public Plugin
 {
     public:
         EnlargedRenderer() throw();
@@ -46,21 +47,33 @@ class EnlargedRenderer :
         virtual void
         init(Application & application) throw();
 
+        virtual void
+        final(Application & application) throw();
+
+        virtual void
+        visit_renderer(BrowserRenderer & browser_renderer) throw();
+
+        virtual void
+        visit_renderer(EnlargedRenderer & enlarged_renderer) throw();
+
+        virtual void
+        visit_renderer(SlideshowRenderer & slideshow_renderer) throw();
+
         virtual void
         render(const PhotoPtr & photo) throw();
 
         virtual void
         render(const PhotoList & photos) throw();
 
-        virtual void
-        final(Application & application) throw();
-
         void
         on_scroll_event(GdkScrollDirection direction) throw();
 
         virtual PhotoList
         get_current_selection() throw();
 
+        virtual TreePathList
+        get_selected_paths() const throw();
+
         virtual void
         present() throw();
 
@@ -87,6 +100,9 @@ class EnlargedRenderer :
         void
         on_action_go_last() throw();
 
+        void
+        on_action_quit_enlarged() throw();
+
         void
         on_action_view_slideshow() throw();
 
